import {extract} from "./Utilities.js";

export interface Open_Food_Facts_item {
    product_name: string,
    image_URLs: Array<string>
}

export class Open_Food_Facts {

    private static readonly _Items = new Array();
    private static readonly _Path = "/api/v0/product/";
    private static readonly _RequestInit = {
        body: null, // See 'get' below
        headers: {
            'accept': 'application/json', // -> response
            'content-type': 'text/plain' // No need, anyway
        },
        method: 'get'
    };
    private static readonly _Status = "status"; // '0' not found
    private static readonly _URL = "https://ssl-api.openfoodfacts.org";

    static Get_instance(gtin: string) {
        let off = Open_Food_Facts._Items.find(element => {
            return element._gtin === gtin;
        });
        if (off === undefined) {
             window.console.log("Created in 'Open_Food_Facts._Items': " + gtin);
            off = new Open_Food_Facts(gtin);
            Open_Food_Facts._Items.push(off);
        }
        return off;
    }

    private readonly _item_data: Promise<any>;

    constructor(private readonly _gtin: any) {
        this._item_data = new Promise((ready, problem) => { // 'problem' aims at being called when 'Promise' object has difficulty in achieving its job...
            this._get_Open_Food_Facts_item(ready, problem);
        });
    }

    async get_Open_Food_Facts_item() { // So returned type is inferred from 'async'...
        return await this._item_data; // Wait for '_get_item_data' (i.e., 'ready(offi);')
    }

    _get_Open_Food_Facts_item(ready: (offi: Open_Food_Facts_item) => void, problem: Function /*, time_out*/) {
        fetch(`${Open_Food_Facts._URL}${Open_Food_Facts._Path}${this._gtin}.json`, Open_Food_Facts._RequestInit)
            .then(response => {
                response.json().then(result => {
                    window.console.assert(typeof result === 'object');
                    const found = result[Open_Food_Facts._Status];
                    window.console.log(this._gtin + " status: " + result[Open_Food_Facts._Status]);
                    if (found !== undefined && found === 1) {
                        const offi: Open_Food_Facts_item = {
                            product_name: extract(result, "product", "product_name"),
                            image_URLs: [
                                extract(result, "product", "image_url"),
                                extract(result, "product", "image_front_url"),
                                extract(result, "product", "image_small_url")
                            ]
                        };
                        // Remove 'null' and duplicates:
                        offi["image_URLs"] = Array.from(new Set(offi["image_URLs"].filter(image_URL => image_URL !== null)));
                        ready(offi);
                    } else { // Not found
                        const offi: { product_name: string, image_URLs: Array<string> } = {
                            product_name: this._gtin,
                            image_URLs: []
                        };
                        offi['product_name'] = this._gtin;
                        offi['image_URLs'] = [];
                    }
                }).catch(function (error) {
                    problem();
                    window.alert('Error response.json().then: ' + error.message);
                });
            }).catch(function (error) {
                problem();
                window.alert('Error ' + `${Open_Food_Facts._URL}: ` + error.message);
        });
    }
}
