import {Open_Food_Facts_item, Open_Food_Facts} from './Open_Food_Facts.js';

interface Item {
    gtin: string,
    offi: Open_Food_Facts_item
}

class Shopping {

    private static readonly _Display_limit = 10000; // 10 sec.
    private static readonly _Items = new Array();
    private static readonly _Number_of_columns = 3; // 16;
    private static readonly _Number_of_rows = 2; // 12;

    private static readonly _Buffer = new Array();
    private static _Query: HTMLParagraphElement;
    private static readonly _GTIN = /^\d+$/; // Numbers only...
    private static readonly _Query_content = /^[a-z0-9 ]+$/i; // 'Space' included...

    static _Display(item: Item) {
        const shopping: HTMLDivElement = window.document.getElementById("Shopping") as HTMLDivElement;
        const images = shopping.querySelectorAll('[time-out="0"]');
        let image: HTMLImageElement | null;
        let index;
        if (images.length === 0) {
            if (window.confirm("No room on the screen...")) {
                index = Math.floor(Math.random() * Shopping._Number_of_columns * Shopping._Number_of_rows);
                image = shopping.querySelector('[index="' + index + '"]');
            } else
                window.open("https://world.openfoodfacts.org", "Open Food Facts", "_self");
        } else { // There is at least free room on the screen:
            index = Math.floor(Math.random() * images.length);
            image = images[index] as HTMLImageElement;
        }
        window.console.log("Displayed(" + index + "):" + JSON.stringify(item));

        image!.style.opacity = "1";
        image!.setAttribute("time-out", Shopping._Display_limit.toString()); // Number is transformed into string!
        window.setTimeout(() => { // Image aims at disappearing...
            image!.style.opacity = "0.5";
            // Room on screen is released:
            image!.setAttribute("time-out", "0");
        }, Shopping._Display_limit);

        if (item.offi.image_URLs.length === 0) {
            const canvas = window.document.createElement('CANVAS') as HTMLCanvasElement;
            const canvas_context = canvas.getContext('2d');
            canvas_context!.font = "10px Comic Sans MS";
            canvas_context!.fillStyle = "green";
            canvas_context!.fillText(item.offi.product_name, canvas.width / 2, canvas.height / 2);
            canvas_context!.textAlign = "center";
            image!.src = (canvas !== null) ? canvas.toDataURL('image/png') : '';
        } else {
            image!.onload = () => {
                // @ts-ignore
                $('#' + image!.id).blowup({ // https://github.com/paulkr/blowup.js
                    "background": "#F39C12",
                    "width": 250,
                    "height": 250
                });
            };
            image!.src = item.offi.image_URLs[0];
        }
    }

    static _Initialization() {
        // const div = window.document.querySelector('div'); // Get the first '<div>' element...
        // window.alert(div!.constructor); // 'function HTMLDivElement()...'
        // const div_info = window.document.querySelector('div.info'); // Get the first '<div>' element with class 'info'...
        // window.alert(div_info!.constructor); // 'function HTMLDivElement()...'

        Shopping._Search("3250541918013"); // Mayonnaise "MAILLE"...
        Shopping._Search("3560070478781"); // Pain d'épices...
        Shopping._Search("8710637105697"); // Wraps de blé...
        // Shopping._Search("20023966"); // Cassoulet pu porc...
        Shopping._Search("3017620422003"); // Nutella...
        Shopping._Search("8410076472953"); // 'product_name === null' -> autre Wraps
        Shopping._Search("3664346325585"); // Bonbons "Red'DINGUE"

        // Because of: 'window.addEventListener('DOMContentLoaded', Shopping._Initialization);'
        window.console.assert(window.document.readyState === 'interactive');

        /**
         * HTML stuff...
         */
        /** For test only (to be removed later on in 'index.html'): */
        Shopping._Query = window.document.getElementById('Query') as HTMLParagraphElement;
        const shopping = window.document.getElementById('Shopping') as HTMLDivElement;
        window.console.assert(shopping !== null, "'shopping === null', why?");

        shopping.style.display = 'grid';
        // '"suppressImplicitAnyIndexErrors": true,' required:
        shopping.style['grid-template-columns'] = "repeat(" + Shopping._Number_of_columns + ", 1fr)";
        shopping.style['grid-template-rows'] = "fit-content(" + (100 / Shopping._Number_of_rows) + "%)";
        shopping.style['align-items'] = 'center'; /* Value of 'align-self' for children */
        shopping.style['justify-items'] = 'flex-start'; /* Value of 'justify-self' for children */

        window.addEventListener("END", custom_event => {
            window.console.assert((<CustomEvent>custom_event).detail.i === Shopping._Number_of_columns * Shopping._Number_of_rows - 1);
            // Wait for all 'shopping.appendChild(image);' finished:
            // Filtering by CSS style value is impossible...
            // Get all 'HTMLImageElement' objects that have a 'style' attribute:
            const images = shopping.querySelectorAll('img[style]');
            // Untrue:
            // window.console.assert(images.length === Shopping._Number_of_columns * Shopping._Number_of_rows);
        });

        for (let i = 0; i < Shopping._Number_of_columns * Shopping._Number_of_rows; i++) {
            let image = new Image;
//            image.addEventListener('click', (event) => { // Unused...
//                window.dispatchEvent(new CustomEvent('Validate', {detail: {index: i}}));
//            });
            image.style.height = (100 / (Shopping._Number_of_rows)) + "vh";
            image.setAttribute("id", "Shopping_image" + i);
            image.setAttribute("index", i.toString()); // Number is transformed into string!
            image.style.opacity = "0.5";
            image.setAttribute("time-out", "0");
            image.style.width = (100 / Shopping._Number_of_columns) + "vw";
            image.onload = () => {
                shopping.appendChild(image); // Images are stored in a random way...
                if (i === Shopping._Number_of_columns * Shopping._Number_of_rows - 1)
                    window.dispatchEvent(new CustomEvent('END', {detail: {i: i}}));
            };
            image.src = './img/Franck.jpg'; // Not optimal...
        }
    }

    static _Handle(event: KeyboardEvent) {
        if (event.defaultPrevented)
            window.console.log("'event.preventDefault()' already done..."); // By construction, propagation goes on...
        /**
         * Test
         */
        // window.console.log(`${event.key}` + ' <-event.key - event.code-> ' + `${event.code}`);
        // window.console.log('match: ' + event.key.match(Shopping._Query_content));
        if (Shopping._Query !== null)
            Shopping._Query.innerHTML += `${event.key}`; // Caution: raw display...
        /**
         * End of test
         */
        if (event.code === 'Backspace')
            Shopping._Buffer.pop();
        if (event.code === 'Escape' || event.code === 'Enter') {
            Shopping._Buffer.length = 0;
            if (window.confirm("Erase screen?"))
                Shopping._Query.innerHTML = "";
        }
        if (Shopping._Query_content.test(event.key) && event.key.length === 1) { // Test again regular expression...
            let last = Shopping._Buffer.pop();
            if (event.code === 'Space') {
                if (last !== undefined) {
                    if (!last.includes(" ")) // 'includes' tests equality as well...
                        Shopping._Buffer.push(last); // Re-inject 'last' at last position
                    Shopping._Buffer.push(event.key);
                }
            } else { // Something else that 'Space'...
                if (last === undefined)
                    Shopping._Buffer.push(event.key);
                else {
                    Shopping._Buffer.push(last); // Re-inject 'last' at last position
                    Shopping._Buffer.push(event.key);
                    if (!last.includes(" "))  // 'includes' tests equality as well...
                        Shopping._Search(Shopping._Buffer.join(''));
                }
            }
        }
//        if (Shopping._Query !== null)
//            Shopping._Query.innerHTML = Shopping._Buffer.join(''); // Nice display...
    }

    static _Search(query: string) {
        window.console.info("Attempt to search: " + query);
        const index: number = Shopping._Items.findIndex((item) =>
            item.gtin.toLowerCase().indexOf(query.toLowerCase()) !== -1
                ? true
                : item.offi.product_name === null
                    ? false
                    : item.offi.product_name.toLowerCase().indexOf(query.toLowerCase()) !== -1
                        ? true
                        : false);
        if (index === -1) { // Not found...
            if (Shopping._GTIN.test(query)) // Only numbers in GTIN...
                Open_Food_Facts.Get_instance(query).get_Open_Food_Facts_item().then((offi: Open_Food_Facts_item) => {
                    const item = {gtin: query, offi: offi};
                    if (item.offi.image_URLs.length !== 0) {
                        Shopping._Items.push(item);
                        Shopping._Display(item);
                    }
                });
        }
    }
}

window.addEventListener('DOMContentLoaded', Shopping._Initialization);
window.addEventListener('keyup', Shopping._Handle); // https://developer.mozilla.org/en-US/docs/Web/Events/keyup
window.onpopstate = (event) => {
    window.alert('onpopstate');
};

